var spawn = require('child_process').spawn
var path = require('path')
var tmp = require('tmp')
var rimraf = require('rimraf')
var merge = require('merge')
var split = require('split')
var createDebug = require('debug')
var argv = require('yargs')
  .describe('storage', 'location where snapshots and profiles may be stored')
  .alias('s', 'storage')
  .argv

var port = process.env.NSOLID_CONSOLE_PORT || 3000

var storagePath, tmpobj

if (!argv.storage) {
  tmpobj = tmp.dirSync({ unsafeCleanup: true })

  storagePath = tmpobj.name
} else {
  storagePath = path.resolve(process.cwd(), argv.storage)
}

// Override `debug`'s default so buffers become strings
createDebug.coerce = function coerce (val) {
  if (val instanceof Error) return val.stack || val.message
  return val.toString()
}

process.on('uncaughtException', cleanupAndExit)

process.on('SIGINT', cleanupAndExit)
process.on('SIGTERM', cleanupAndExit)
process.on('beforeExit', cleanupAndExit)

function cleanupAndExit () {
  if (tmpobj) {
    try {
      rimraf.sync(tmpobj.name)
    } catch (e) {

    }
    tmpobj = null
  }
}

function debugWrap (proc, logger) {
  proc.stdout.pipe(process.stdout)
  proc.stderr.pipe(split()).on('data', logger)
  proc.on('close', logger.bind(null, 'closed with code'))
  return proc
}

debugWrap(spawn('etcd', ['--cors=*'], {}), createDebug('etcd'))

debugWrap(spawn('npm', ['run', 'proxy', '--loglevel=silent'], {}), createDebug('proxy'))

debugWrap(spawn('node', [
  'bin/nsolid-console.js',
  '--interval', 1000,
  '--duration', 15,
  '--storage', storagePath,
  '--port', port
], {
  env: merge(true, process.env, {
    env: {
      NODE_ENV: 'development'
    }
  })
}), createDebug('nsolid-console'))

debugWrap(spawn(
  'nsolid',
  ['bin/simulate.js', 'test/simulator/scenarios/noop.js@20'],
  {
    env: merge({
       'NSOLID_APPNAME': 'Shark Attack'
    }, process.env)
  }
), createDebug('scenario:SharkAttack'))

debugWrap(spawn(
  'nsolid',
  ['bin/simulate.js', 'test/simulator/scenarios/leaky-http.js@10'],
  {
    env: merge({
      'NSOLID_APPNAME': 'Beeeeeeeees'
    }, process.env)
  }
), createDebug('scenario:Beeeeeeeees'))
