#!/usr/bin/env node

process.title = 'nsolid-console'

// hack around socket.io debugging BS
process.env.DEBUG = false

var fs = require('fs')
var os = require('os')
var path = require('path')
var rimraf = require('rimraf')

var collect = require('../src/collect.js')
const DEFAULT_STORAGE_DIR = path.join(os.tmpdir(), 'nsolid-console')
var argv = require('yargs')
  .usage('Usage: $0')
  .describe('duration', 'Maximum time to retain graph data in minutes')
  .default('duration', 500)
  .alias('d', 'duration')
  .help('help')
  .alias('help', 'h')
  .describe('interface', 'Networking interface HTTP server will listen on')
  .default('interface', '0.0.0.0')
  .describe('interval', 'Frequency to poll N|Solid Hub in ms')
  .default('interval', 5000)
  .alias('i', 'interval')
  .describe('port', 'Port HTTP server will listen on')
  .default('port', process.env.PORT || 3000)
  .alias('p', 'port')
  .alias('r', 'record')
  .describe('storage', 'Location on disk for storing snapshots & profiles')
  .alias('s', 'storage')
  .default('storage', DEFAULT_STORAGE_DIR)
  .argv

var storagePath = argv.storage

if (storagePath === DEFAULT_STORAGE_DIR) {
  // clean storage dir if not supplied
  try {
    rimraf.sync(storagePath)
  } catch (err) {
    // ignore not found errs
    if (err.code !== 'ENOENT') onError(err)
  }
} else {
  storagePath = path.resolve(process.cwd(), storagePath)
}

var pollingEmitter = collect.poll(argv.interval, argv.duration)

function writeToDisk (obj) {
  fs.appendFile(argv.record, JSON.stringify(obj) + '\n', function (err) {
    if (err) onError(err)
  })
}

if (argv.record) {
  pollingEmitter.on('output', writeToDisk)
}

collect.createServer(argv.port, argv.interface, storagePath, pollingEmitter, function (err) {
  if (err) return onError(err)
  const listening = this.address()
  console.info('Server running on http://%s:%s', listening.address, listening.port)
}, onError)

function onError (err) {
  if (!err) return
  console.error(err.stack || err)

  // add helpful suggestions here
  if (err.code === 'EACCES') {
    console.error('\nHave you configured a writable --storage directory?')
  }

  console.error('\nRun `nsolid-console --help` for command-line help.')
  process.exit(1)
}
