"use strict";

const http = require("http")
const EventEmitter = require("events").EventEmitter
const inherits = require("util").inherits
const terminus = require("terminus")

module.exports = Mock

function Mock() {
  if (!(this instanceof Mock)) {
    return new Mock()
  }
  console.log("my pid: %s", process.pid)
  EventEmitter.call(this)
  let self = this

  let expect = 4
  function readyCheck() {
    if (--expect <= 0) {
      console.log("etcd mock at http://127.0.0.1:%s", self.etcdPort)
      console.log(self.registry)
      self.emit("ready")
    }
  }
  self.agents = []
  self.registry = []

  function startAgent(id) {
    let server = http.createServer(function onReq(req, res) {
      let collect = terminus.concat(function getBody(body) {
        let command = /^\/(\w+)\/?/.exec(req.url)[1]
        if (command == "slow") {
          return setTimeout(function later() {
            return res.end("SUPER SLOW")
          }, 11000)
        }
        let content = (body.length) ? " <" + JSON.parse(body.toString()) + ">" : ""
        if (id == 0) {
          // Always error status
          res.statusCode = 503
          return res.end('"NOPE (error)"')
        }
        return res.end('"PONG ' + command + content + '"')
      })
      req.pipe(collect)
    })
    server.listen(0, "127.0.0.1", function onListen() {
      self.agents.push(server)
      self.registry.push({
        id: id,
        app: (id == 0) ? "broke" : "mock",
        address: "127.0.0.1:" + server.address().port,
        pid: process.pid,
        hostname: "foobar.hostname"
      })
      readyCheck()
    })
  }

  // mock up some agents
  let agentCount = expect - 1
  for (let i = 0; i < agentCount; i++) {
    startAgent(i)
  }

  // mock etcd
  let etcdServer = http.createServer(function onReq(req, res) {
    function toNodes(app) {
      return self.registry
        .filter(function (node) { return node.app == app})
        .map(function (node) {
          return {value: JSON.stringify(node)}
        })
    }
    let content = null
    // handle stuff...
    if (req.url.match(/nsolid\/\?/)) {
      content = {
        node: {
          nodes: [
            {
              key: "/nsolid/broke",
              nodes: toNodes("broke")
            },
            {
              key: "/nsolid/mock",
              nodes: toNodes("mock")
            }
          ]
        }
      }
    }
    if (req.url.match(/nsolid\/mock\?/)) {
      content = {
        node: {
          nodes: [
            {
              key: "/nsolid/mock",
              nodes: toNodes("mock")
            }
          ]
        }
      }
    }
    if (req.url.match(/nsolid\/broke\?/)) {
      content = {
        node: {
          nodes: [
            {
              key: "/nsolid/broke",
              nodes: toNodes("broke")
            }
          ]
        }
      }
    }
    if (req.url.match(/nsolid\/nope\?/)) {
      content = {
        node: {
          nodes: [ ]
        }
      }
    }
    // lookup
    var routeRe = /nsolid\/(\w+)\/(\d)/
    if (req.url.match(routeRe)) {
      let match = routeRe.exec(req.url)
      let app = match[1]
      let id = match[2]
      let host = self.registry.filter(function (host) {
        return host.app == app && host.id == id
      })
      content = {
        node: {
          value: JSON.stringify(host[0])
        }
      }
    }
    res.end(JSON.stringify(content))
  })
  etcdServer.listen(0, "127.0.0.1", function onListen() {
    self.etcd = etcdServer
    // get port and send it back in the callback
    self.etcdPort = etcdServer.address().port
    readyCheck()
  })
}
inherits(Mock, EventEmitter)

Mock.prototype.stop = function stop() {
  console.log("stopping")
  if (this.etcd) {
    this.etcd.close()
  }
  this.agents.forEach(function eaAgent(agent) {
    agent.close()
  })
}
