var createArrayStore = require('./array-store')
var addProcess = require('./add-process')

var spec = require('./graphs/spectrogram')
var computeSparklines = require('./graphs/sparkline')

module.exports = buildResponse

// function getSpectrogramInput (application) {
//   var output = [];
//   forEach(application.processes, function (process) {
//     output.push({
//       heapUsed: process.memory.heapUsed
//       heapTotal: process.memory.heapTotal
//     });
//   }
//   return output;
// }

function buildResponse (processes, hosts, pollInterval, duration) {
  var getApp = createArrayStore({
    hosts: [],
    snapshots: [],
    graphs: {
      rss: [],
      heapTotal: [],
      heapUsed: [],
      spectrogram: [],
      spiffygram: []
    },
    processes: []
  })

  // given the fact that we _may_ have a set of valid proceses
  // loop through all of the process and ensure that they are
  // valid and can be linked to a valid host
  //
  // valid processes mean that they have received both info and process_stats responses
  // valid hosts mean that they have received both system_info and system_stats responses
  //
  // once we have this information we can build the array of applications.  This is done
  // at runtime and is not stored because doing so would add state without any discernible
  // advantage
  Object.keys(processes).forEach(function (key) {
    var processResponse = processes[key]
    var processId = processResponse.meta.id
    var process = processResponse.reply

    process.app = processResponse.meta.app // TODO: fix this by passing app into `addProcess`

    // we need to ensure that the intersection between
    // (info ∩ process_stats) & (system_info ∩ system_stats)
    // is valid. if not then we can drop the process, and
    // hopefully pick it up on the next polling cycle.

    if (hosts && process && hosts[processId] && hosts[processId]) {
      var host = hosts[processId]

      var application = addProcess(process, host, getApp)

      // only insert one instance of the host object per application.
      if (application.hosts.indexOf(host) === -1) {
        application.hosts.push(host)
      }
    }
  })

  // return the array of _computed_ applications w/ valid (complete) processes
  return getApp.array.map(function (application) {
    computeSparklines(application, pollInterval, duration)

    spec.computeRSSSpectrogram(application, 25, 10)
    spec.computeCPUSpectrogram(application, 25, 10)
    spec.computeSpiffygram(application, 5, 5)

    return application
  })
}
