var d3 = require('d3')

var GB = 1024 * 1024 * 1024

// create a width * height 2d array
function Array2D(width, height) {
  var out = []
  for (var i = 0; i < width; i++) {
    out[i] = []
    for (var j = 0; j < height; j++) {
      out[i][j] = 0
    }
  }
  return out
}

function computeCPUSpectrogram (application, width, height) {
  var graph = application.graphs.CPUSpectrogram = application.graphs.CPUSpectrogram || Array2D(width, height)
  var processes = application.processes

  // input ranges
  var min = 0
  var max = 100

  var data = d3.layout.histogram()
  .frequency(false)
  .bins(d3.range(min, max, max / height).concat(max))
  .value(function(p) { return p.cpu })
  (processes)

  // update columns
  graph.push(data.map(function(d) {return d.y}))
  graph.splice(0, 1)
  application.graphs.CPUSpectrogram = graph
}

function computeRSSSpectrogram (application, width, height) {
  var graph = application.graphs.RSSSpectrogram = application.graphs.RSSSpectrogram || Array2D(width, height)
  var processes = application.processes
  var totalRSS = Math.max(d3.sum(processes, function(p) {return p.memory.rss}), GB)

  // input ranges
  var min = 0
  var max = 1

  var data = d3.layout.histogram()
  .frequency(false)
  .bins(d3.range(min, max, max / height).concat(max))
  .value(function(p) { return p.memory.rss / totalRSS })
  (processes)

  // update columns
  graph.push(data.map(function(d) {return d.y}))
  graph.shift()
  application.graphs.RSSSpectrogram = graph
}

function computeSpiffygram (application, width, height) {
  var grid = Array2D(width, height)

  var processes = application.processes

  var largestCpu = d3.max(processes, function(p) {return p.cpu})
  var totalRSS = Math.max(d3.sum(processes, function(p) {return p.memory.rss}), GB)
  var incrementor = 1/processes.length

  processes.forEach(function (g) {

    var Xratio = Math.min(g.cpu / 100, .99)
    var Yratio = Math.min(g.memory.rss / totalRSS, .99)

    var x = Math.floor(Xratio * width)

    // flip the Y-Axis so small values are on the bottom
    var y = (height - 1) - Math.floor(Yratio * height)
    var gridX = grid[x];
    if (gridX && y < gridX.length) {
      gridX[y] += incrementor;
    }
  })

  application.graphs.spiffygram = grid
}

exports.computeRSSSpectrogram = computeRSSSpectrogram;
exports.computeCPUSpectrogram = computeCPUSpectrogram;
exports.computeSpiffygram = computeSpiffygram
