var split = require('split')
var spawn = require('child_process').spawn

module.exports = query
module.exports.json = json

function json (s) {
  try {
    return JSON.parse(s.replace(/\\/g, '\\\\'))
  } catch (e) {
    return { error: e }
  }
}

function query (path, cmd, args, fn) {
  if (typeof args === 'function') {
    fn = args
    args = []
  } else {
    var keys = Object.keys(args)
    args = keys.map(function (key) {
      return '--' + key + '=' + args[key]
    })
  }

  var child = spawn(path, [cmd].concat(args), {
    stdio: 'pipe',
    env: process.env
  })

  child.stderr.pipe(process.stderr)

  child.on('error', fn)

  var res = []

  child.stdout.pipe(split()).on('data', function (line) {
    if (line) {
      var obj = json((line || '').toString())
      if (!obj.error) {
        res.push(obj)
      }
    }
  })

  child.on('exit', function (code) {
    if (code) {
      return fn(new Error('query exited with code: ' + code))
    }
    fn(null, res)
  })
}
