#nsolid-listener


A very simple REST-like http server API for N|Solid.

```js
var Listener = require("nsolid-listener")
var listener = Listener()

listener.on("ping", function (req, rpc, next) {
  next(null, "OK")
})
```

#API

##`var listener = require("nsolid-listener")(socket)`

Create a new listener with the specified listening on the specified `socket`

`socket` can be:
* `null`: bind a random available port
* a port number: bind this specified port
* an address: e.g. 127.0.0.1:8000 to bind only this interface at port 8000
* an ip address: IPv4 or IPv6 that is in the listed interfaces for this host
* an address w/ wildcard port: e.g. 127.0.0.1:? to bind a random port with this interface

##`var addressInfo = listener.getAddress()`

Lists the port and ips this listener is listening on. When no interface is specified, it will list all public interfaces as it cannot know what a remote server can connect to.

```js
var addressInfo = listener.getAddress()
console.log(addressInfo)
{ port: 28282,
  ips:
   [ '10.0.0.27',
     '2601:9:7b83:8fb9:443d:bfbd:4630:1f07',
     '2601:9:7b83:8fb9:3602:86ff:fe1a:144f',
     '2601:9:7b83:8fb9::9514',
     'fe80::3602:86ff:fe1a:144f' ] }
```


##Events

###Listening

Emit when the server is ready to be listened to. This prevents a "listening" RPC event from working correctly.

###*event*

Emit whenever a RPC client connects. RPC objects can have three properties:
* command
* id
* data

For a POST to /ping/123 with POST data "FOO", the RPC object would be:
```js
{
  command: "ping",
  id: 123,
  data: "FOO"
}
```
If there is a listener bound, it will call the listener with the arguments `onEvent(req, rpcObject, next)`

You must call `next` to continue the route. Calling `next(err)` will send an error, and `next(null, data)` will JSON serialize the `data` and send it back to the RPC client.


###wildcard

To bind a "wildcard" event, you can listen for the "\*" event, it behaves like any other event, except it will only be emit if an RPC command does not have a listener bound.
