"use strict";

var Listener = require("nsolid-listener")
var Routing = require("./router")
var debug = require("util").debuglog("nsolid")
var conf = require("rc")("nsolid-proxy")

if (conf.port == null) {
  throw new Error("Could not find configuration for port")
}
var listener = new Listener(conf.port)

listener.on("listening", function startRouter() {
  var registryUri = process.env.REGISTRY || conf.registry || ""
  debug("Connection to registry at: ", registryUri)
  // TBD: eventually support multiple registries based on url
  registryUri = registryUri.replace(/^etcd:\/\//, "")
  var options = {
    etcd_host: registryUri,
    denied: conf.denied,
    broadcast_approved: conf.broadcast_approved
  }
  var router = new Routing(options)

  // Hub commands
  listener.on("ls", router.list.bind(router))

  // Everything else -- proxy
  listener.on("*", router.proxy.bind(router))
})

function gracefulShutdown(err) {
  if (err instanceof Error) {
    console.error("Graceful shutdown triggered due to error")
    console.error(err)
    console.error(err.stack)
  }
  console.log("Shutting down hub")
  listener.server.close()
  var wait = setTimeout(function () {
    console.error("Forcing shutdown after wait")
    process.exit(1)
  }, 5000)
  wait.unref()
}

process.on("uncaughtException", gracefulShutdown)
process.on("SIGINT", gracefulShutdown)
process.on("SIGTERM", gracefulShutdown)
process.on("SIGHUP", gracefulShutdown)
